@@ -20,10 +20,7 @@
 #define MyHTML_MYOSI_H
 #pragma once
 
-#ifdef __cplusplus
-extern "C" {
-#endif
-
+#include <stdbool.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <memory.h>
@@ -35,6 +32,18 @@ extern "C" {
 #include <windows.h>
 #endif
 
+#if defined(_MSC_VER)
+#  define MyHTML_DEPRECATED(func, message) __declspec(deprecated(message)) func
+#elif defined(__GNUC__) || defined(__INTEL_COMPILER)
+#  define MyHTML_DEPRECATED(func, message) func __attribute__((deprecated(message)))
+#else
+#  define MyHTML_DEPRECATED(func, message) func
+#endif
+
+#ifdef __cplusplus
+extern "C" {
+#endif
+
 #ifdef DEBUG_MODE
     #define MyHTML_DEBUG(__format__, ...)      \
         myhtml_print(stderr, "DEBUG: "__format__"\n", ##__VA_ARGS__)
@@ -57,7 +66,11 @@ extern "C" {
             sizeof(__strcn__) * __myhtml__->__sizen__);                                     \
     }
 
-typedef enum {myfalse = 0, mytrue = 1} mybool_t;
+
+typedef enum {
+    myfalse = 0,
+    mytrue  = 1
+} MyHTML_DEPRECATED(mybool_t, "use bool");
 
 // encoding
 // https://encoding.spec.whatwg.org/#the-encoding
@@ -371,7 +384,6 @@ typedef myhtml_token_attr_t myhtml_tree_attr_t;
 typedef struct myhtml_collection myhtml_collection_t;
 typedef struct myhtml myhtml_t;
 
-
 // parser state function
 typedef size_t (*myhtml_tokenizer_state_f)(myhtml_tree_t* tree, mythread_queue_node_t* qnode, const char* html, size_t html_offset, size_t html_size);
 
@@ -379,7 +391,7 @@ typedef size_t (*myhtml_tokenizer_state_f)(myhtml_tree_t* tree, mythread_queue_n
 typedef void (*mythread_f)(mythread_id_t thread_id, mythread_queue_node_t *qnode);
 
 // parser insertion mode function
-typedef mybool_t (*myhtml_insertion_f)(myhtml_tree_t* tree, myhtml_token_node_t* token);
+typedef bool (*myhtml_insertion_f)(myhtml_tree_t* tree, myhtml_token_node_t* token);
 
 void * mymalloc(size_t size);
 void * myrealloc(void* dst, size_t size);
